diff -ubBr nrpe-1.5/common/common.h nrpe-1.5.new/common/common.h
--- nrpe-1.5/common/common.h	Mon Jun  3 19:08:17 2002
+++ nrpe-1.5.new/common/common.h	Mon Dec 23 17:31:55 2002
@@ -46,6 +46,7 @@
 
 #define NRPE_HELLO_COMMAND      "_NRPE_CHECK"
 
+#define MAX_ARGUMENTS_LENGTH 2048
 
 /**************** PACKET STRUCTURE DEFINITION **********/
 
@@ -53,6 +54,7 @@
 #define	RESPONSE_PACKET		2		/* id code for a packet containing a response */
 
 #define NRPE_PACKET_VERSION_1	1		/* packet version identifier (in case packet type changes in future versions) */
+#define NRPE_PACKET_VERSION_2	2
 
 #define MAX_PACKETBUFFER_LENGTH	1024		/* max amount of data we'll send in one query/response */
 
@@ -62,4 +64,6 @@
 	int result_code;
 	int buffer_length;
 	char buffer[MAX_PACKETBUFFER_LENGTH];
-        }packet;
+	char argv[MAX_ARGUMENTS_LENGTH];  /* the arguments to be used */
+	int argc;
+} packet;
diff -ubBr nrpe-1.5/src/check_nrpe.c nrpe-1.5.new/src/check_nrpe.c
--- nrpe-1.5/src/check_nrpe.c	Mon Jun  3 19:08:17 2002
+++ nrpe-1.5.new/src/check_nrpe.c	Fri Dec 27 09:39:51 2002
@@ -1,22 +1,22 @@
 /********************************************************************************************
- *
- * CHECK_NRPE.C - NRPE Plugin For Nagios
- * Copyright (c) 1999-2002 Ethan Galstad (nagios@nagios.org)
- * License: GPL
- *
- * Last Modified: 06-03-2002
- *
- * Command line: CHECK_NRPE <host_address> [-p port] [-c command] [-wt warn_time] \
- *                          [-ct crit_time] [-to to_sec]
- *
- * Description:
- *
- * This plugin will attempt to connect to the NRPE daemon on the specified server and port.
- * The daemon will attempt to run the command defined as [command].  Program output and
- * return code are sent back from the daemon and displayed as this plugin's own output and
- * return code.
- *
- ********************************************************************************************/
+*
+* CHECK_NRPE.C - NRPE Plugin For Nagios
+* Copyright (c) 1999-2002 Ethan Galstad (nagios@nagios.org)
+* License: GPL
+*
+* Last Modified: 06-03-2002
+*
+* Command line: CHECK_NRPE <host_address> [-p port] [-c command] \
+*                          [-t to_sec] [-a args] 
+*
+* Description:
+*
+* This plugin will attempt to connect to the NRPE daemon on the specified server and port.
+* The daemon will attempt to run the command defined as [command].  Program output and
+* return code are sent back from the daemon and displayed as this plugin's own output and
+* return code.
+*
+********************************************************************************************/
 
 #include "../common/common.h"
 #include "../common/config.h"
@@ -27,10 +27,13 @@
 int server_port=DEFAULT_SERVER_PORT;
 char server_name[MAX_HOST_ADDRESS_LENGTH];
 
-char query_string[MAX_PACKETBUFFER_LENGTH]=DEFAULT_NRPE_COMMAND;;
+/* char query_string[MAX_PACKETBUFFER_LENGTH]=DEFAULT_NRPE_COMMAND;*/
+char *query_string = DEFAULT_NRPE_COMMAND;
 int socket_timeout=DEFAULT_SOCKET_TIMEOUT;
-
-
+int use_v2 = FALSE;
+int num_args = 0;
+char *argv_to_pass = NULL;
+int argv_idx = 0;
 
 int process_arguments(int,char **);
 void alarm_handler(int);
@@ -49,6 +52,10 @@
 
 	result=process_arguments(argc,argv);
 
+    /*
+    printf("Args: version: %d, query_string %s, numargs %d, args %s\n",
+        (use_v2)?2:1, query_string, num_args, argv_to_pass);
+    */
 	if(result!=OK){
 
 		printf("Incorrect command line arguments supplied\n");
@@ -59,8 +66,8 @@
 		printf("Last Modified: %s\n",MODIFICATION_DATE);
 		printf("License: GPL\n");
 		printf("\n");
-		printf("Usage: %s <host_address> [-p port] [-c command] [-wt warn_time]\n",argv[0]);
-		printf("          [-ct crit_time] [-to to_sec]\n");
+        printf("Usage: %s <host_address> [-p port] [-c command]\n",argv[0]);
+        printf("          [-t to_sec] [-V version] [-a arg]\n");
 		printf("\n");
 		printf("Options:\n");
 		printf(" <host_address> = The IP address of the host running the NRPE daemon\n");
@@ -68,6 +75,8 @@
 		printf(" [command]      = The name of the command that the remote daemon should run\n");
 		printf(" [to_sec]       = Number of seconds before connection attempt times out.\n");
 		printf("                  Default timeout is %d seconds\n",DEFAULT_SOCKET_TIMEOUT);
+        printf(" [version]      = Version to use (version 2 allows for arguments).\n");
+        printf(" [arg]          = Argument string to pass.  Can be double quoted for multiword argument.\n");        
 		printf("\n");
 		printf("Note:\n");
 		printf("This plugin requires that you have the NRPE daemon running on the remote host.\n");
@@ -97,10 +106,24 @@
 		/* send the query packet */
 		bzero(&send_packet,sizeof(send_packet));
 		send_packet.packet_type=htonl(QUERY_PACKET);
-		send_packet.packet_version=htonl(NRPE_PACKET_VERSION_1);
+        if(use_v2)
+            send_packet.packet_version = htonl(NRPE_PACKET_VERSION_2);
+        else
+            send_packet.packet_version = htonl(NRPE_PACKET_VERSION_1);
+
 		send_packet.buffer_length=htonl(strlen(query_string));
 		strcpy(&send_packet.buffer[0],query_string);
 
+        /* set up the arguments if necessary */
+        if(use_v2)
+        {
+            int len;
+            send_packet.argc = num_args;
+            strcpy(send_packet.argv, argv_to_pass);
+            /* printf("Sending argument '%s'\n", send_packet.argv); */
+            
+        }
+
 		bytes_to_send=sizeof(send_packet);
 		rc=sendall(sd,(char *)&send_packet,&bytes_to_send);
 
@@ -155,65 +178,74 @@
 	alarm(0);
 
 	return result;
-        }
+}
 
 
 
 /* process command line arguments */
 int process_arguments(int argc, char **argv){
-	int x;
-
+    /* int x; */
+    int c, x;
 
 	/* no options were supplied */
 	if(argc<2)
 		return ERROR;
 
-	/* first option is always the server name/address */
-	strncpy(server_name,argv[1],sizeof(server_name)-1);
-	server_name[sizeof(server_name)-1]='\x0';
-
-	/* process all remaining arguments */
-	for(x=3;x<=argc;x++){
-
-		if(!strcmp(argv[x-1],"-c")){
-			if(x<argc){
-				strncpy(query_string,argv[x],sizeof(query_string)-1);
-				query_string[sizeof(query_string)-1]='\x0';
-				x++;
-			        }
-			else
-				return ERROR;
-		        }
-		else if(!strcmp(argv[x-1],"-p")){
-			if(x<argc){
-				server_port=atoi(argv[x]);
-				x++;
-			        }
-			else
-				return ERROR;
-		        }
-		else if(!strcmp(argv[x-1],"-to")){
-			if(x<argc){
-				socket_timeout=atoi(argv[x]);
-				if(socket_timeout<=0)
-					return ERROR;
-				x++;
-			        }
-			else
-				return ERROR;
+    /* read the server name first */
+    strncpy(server_name, argv[1], sizeof(server_name)-1);
+
+    /* now remove them from the array */
+    argc--;
+    argv++;
+
+    while (1)
+    {
+        c = getopt (argc, argv, "c:p:t:V:a:n:");
+
+        if(c == -1 || c == EOF)
+            break;
+
+        switch (c)
+        {
+        case 'c':
+            query_string = optarg;
+            break;
+        case 'p':
+            server_port = atoi(optarg);
+            break;
+        case 't':
+            socket_timeout = atoi(optarg);
+            break;
+        case 'V':
+            x = atoi(optarg);
+            if(x < 1 || x > 2)
+            {
+                printf("version must be between 1 and 2\n");
+                return ERROR;
+            }
+            if(x == 2)
+                use_v2 = TRUE;
+            break;
+        case 'n':
+            num_args = atoi(optarg);
+            break;
+        case 'a':
+            argv_to_pass = optarg;
+            break;
 		        }
-		else
-			return ERROR;
 	        }
 
+    if((argv_to_pass != NULL) && (num_args == 0))
+        num_args = 1;
+
 	return OK;
-        }
 
+}
 
 
-void alarm_handler(int sig){
 
+void alarm_handler(int sig)
+{
 	printf("CHECK_NRPE: Socket timeout after %d seconds.\n",socket_timeout);
-
 	exit(STATE_CRITICAL);
-        }
+}
diff -ubBr nrpe-1.5/src/nrpe.c nrpe-1.5.new/src/nrpe.c
--- nrpe-1.5/src/nrpe.c	Mon Jun  3 19:08:17 2002
+++ nrpe-1.5.new/src/nrpe.c	Fri Dec 27 09:39:47 2002
@@ -36,8 +36,11 @@
 int drop_privileges(char *,char *);
 void free_memory(void);
 int is_an_allowed_host(char *);
+int handle_v1_command(char *, char **, int);
+int handle_v2_command(char *, char *, int , char **, int ); 
 
 int my_system(char *,int,int *,char *,int);            	/* executes a command via popen(), but also protects against timeouts */
+int my_system_v2(int, char *,char *,int,int *,char *,int);
 void my_system_sighandler(int);				/* handles timeouts when executing commands via my_system() */
 
 
@@ -171,7 +174,7 @@
 	/* We are now running in daemon mode, or the connection handed over by inetd has
 	   been completed, so the parent process exits */
         return STATE_OK;
-	}
+}
 
 
 
@@ -281,7 +284,7 @@
 	fclose(fp);
 
 	return OK;
-	}
+}
 
 
 
@@ -305,7 +308,7 @@
 		syslog(LOG_DEBUG,"Added command[%s]=%s\n",command_name,command_line);
 
 	return OK;
-        }
+}
 
 
 
@@ -318,7 +321,7 @@
 			return temp_command;
 
 	return NULL;
-        }
+}
 
 
 
@@ -490,20 +493,18 @@
 	syslog(LOG_NOTICE,"Terminating");
 
 	return;
-	}
+}
 
 
 
 /* handles a client connection */
 void handle_connection(int sock){
-	command *temp_command;
 	packet receive_packet;
 	packet send_packet;
 	int bytes_to_send;
 	int bytes_to_recv;
-	char buffer[MAX_INPUT_BUFFER];
+    char *buffer = NULL;
 	int result=STATE_OK;
-	int early_timeout=FALSE;
 	int rc;
 
 
@@ -536,7 +537,10 @@
 	        }
 
 	/* make sure this is the right type of packet */
-	if(ntohl(receive_packet.packet_type)!=QUERY_PACKET || ntohl(receive_packet.packet_version)!=NRPE_PACKET_VERSION_1){
+	if(ntohl(receive_packet.packet_type)!=QUERY_PACKET ||
+        ( ntohl(receive_packet.packet_version)!=NRPE_PACKET_VERSION_1 && ntohl(receive_packet.packet_version)!=NRPE_PACKET_VERSION_2 )
+      )
+    {
 
 		/* log error to syslog facility */
 		syslog(LOG_ERR,"Received invalid packet from client, bailing out...");
@@ -548,11 +552,14 @@
 	if(debug==TRUE)
 		syslog(LOG_DEBUG,"Host is asking for command '%s' to be run...",receive_packet.buffer);
 
+    /* make all the space necessary for the output */
+    buffer = calloc(1, MAX_INPUT_BUFFER*sizeof(char));
+
 	/* if this is the version check command, just spew it out */
 	if(!strcmp(&receive_packet.buffer[0],NRPE_HELLO_COMMAND)){
 
-		snprintf(buffer,sizeof(buffer),"NRPE v%s",PROGRAM_VERSION);
-		buffer[sizeof(buffer)-1]='\x0';
+		snprintf(buffer,MAX_INPUT_BUFFER,"NRPE v%s",PROGRAM_VERSION);
+		buffer[MAX_INPUT_BUFFER-1]='\x0'; 
 
 		/* log info to syslog facility */
 		if(debug==TRUE)
@@ -563,47 +570,26 @@
 
 	/* find the command we're supposed to run */
 	else{
-		temp_command=find_command(receive_packet.buffer);
-		if(temp_command==NULL){
-
-			snprintf(buffer,sizeof(buffer),"NRPE: Command '%s' not defined",receive_packet.buffer);
-			buffer[sizeof(buffer)-1]='\x0';
-
-			/* log error to syslog facility */
-			if(debug==TRUE)
-				syslog(LOG_DEBUG,"%s",buffer);
-
-			result=STATE_CRITICAL;
+        if(ntohl(receive_packet.packet_version) == NRPE_PACKET_VERSION_1)
+            result = handle_v1_command(receive_packet.buffer, &buffer, MAX_INPUT_BUFFER);
+        else if(ntohl(receive_packet.packet_version) == NRPE_PACKET_VERSION_2)
+        {
+            result = handle_v2_command(receive_packet.buffer, 
+                                       receive_packet.argv, 
+                                       receive_packet.argc,
+                                       &buffer,
+                                       MAX_INPUT_BUFFER);
 	                }
 
-		else{
-
-			/* log info to syslog facility */
-			if(debug==TRUE)
-				syslog(LOG_DEBUG,"Running command: %s",temp_command->command_line);
-
-			/* run the command */
-			strcpy(buffer,"");
-			result=my_system(temp_command->command_line,COMMAND_TIMEOUT,&early_timeout,buffer,sizeof(buffer));
-
-			/* see if the command timed out */
-			if(early_timeout==TRUE)
-				snprintf(buffer,sizeof(buffer)-1,"NRPE: Command timed out after %d seconds\n",COMMAND_TIMEOUT);
-			else if(!strcmp(buffer,""))
-				snprintf(buffer,sizeof(buffer)-1,"NRPE: Unable to read output\n");
-
-			buffer[sizeof(buffer)-1]='\x0';
 
 			/* check return code bounds */
-			if((result<0) || (result>3)){
-
+        if((result<0) || (result>3))
+        {            
 				/* log error to syslog facility */
-				syslog(LOG_ERR,"Bad return code for [%s]: %d", buffer,result);
-
+            syslog(LOG_ERR,"Bad return code for [%s]: %d", *buffer,result);            
 				result=STATE_UNKNOWN;
 			        }
 		        }
-	        }
 
 	/* strip newline character from end of output buffer */
 	if(buffer[strlen(buffer)-1]=='\n')
@@ -629,9 +615,91 @@
 		syslog(LOG_DEBUG,"Return Code: %d, Output: %s",result,buffer);
 
 	return;
+}
+
+/* this executes the version 2 command */
+int handle_v2_command(char *req_comm, char *largv, int largc, char **buffer, int bufsz)
+{
+	command *temp_command;
+    int result;
+    int early_timeout=FALSE;
+    int d;
+
+
+    temp_command=find_command(req_comm);
+    if(temp_command==NULL)
+    {    
+        snprintf(*buffer,bufsz,"NRPE: Command '%s' not defined",req_comm);
+        /* buffer[bufsz-1]='\x0'; */
+        
+        /* log error to syslog facility */
+        if(debug==TRUE)
+            syslog(LOG_DEBUG,"%s",buffer);
+        
+        result = STATE_CRITICAL;
         }
+    else
+    {        
+        /* log info to syslog facility */
+        if(debug==TRUE)
+            syslog(LOG_DEBUG,"Running command (v2): %s",temp_command->command_line);
 
+        /* run the command */
+        strcpy(*buffer,"");
+        /* result=my_system(temp_command->command_line,COMMAND_TIMEOUT,&early_timeout,*buffer,bufsz); */
+        result=my_system_v2(NRPE_PACKET_VERSION_2, temp_command->command_line,largv,
+                            COMMAND_TIMEOUT, &early_timeout, *buffer, bufsz);
 
+        /* see if the command timed out */
+        if(early_timeout==TRUE)
+            snprintf(*buffer,bufsz-1,"NRPE: Command timed out after %d seconds\n",COMMAND_TIMEOUT);
+        else if(!strcmp(*buffer,""))
+            snprintf(*buffer,bufsz-1,"NRPE: Unable to read output\n");
+    }
+    return result;
+}
+
+/* this executes the version 1 command */
+int handle_v1_command(char *req_comm, char **buffer, int bufsz)
+{
+	command *temp_command;
+    int result;
+    int early_timeout=FALSE;
+
+
+    temp_command=find_command(req_comm);
+    if(temp_command==NULL)
+    {    
+        snprintf(*buffer,bufsz,"NRPE: Command '%s' not defined",req_comm);
+        /* buffer[bufsz-1]='\x0'; */
+        
+        /* log error to syslog facility */
+        if(debug==TRUE)
+            syslog(LOG_DEBUG,"%s",buffer);
+        
+        result = STATE_CRITICAL;
+    }    
+    else
+    {
+        /* log info to syslog facility */
+        if(debug==TRUE)
+            syslog(LOG_DEBUG,"Running command (v1): %s",temp_command->command_line);
+        
+        /* run the command */
+        strcpy(*buffer,"");
+        result=my_system(temp_command->command_line,COMMAND_TIMEOUT,&early_timeout,*buffer,bufsz);
+        
+        /* see if the command timed out */
+        if(early_timeout==TRUE)
+            snprintf(*buffer,bufsz-1,"NRPE: Command timed out after %d seconds\n",COMMAND_TIMEOUT);
+        else if(!strcmp(*buffer,""))
+            snprintf(*buffer,bufsz-1,"NRPE: Unable to read output\n");
+        
+        /* buffer[bufsz-1]='\x0'; */
+    }
+
+    return result;
+}
 
 /* checks to see if a given host is allowed to talk to us */
 int is_an_allowed_host(char *connecting_host){
@@ -647,7 +715,7 @@
 	        }
 
 	return 0;
-        }
+}
 
 
 
@@ -662,7 +730,7 @@
 
 	/* so the compiler doesn't complain.. */
 	return;
-        }
+}
 
 
 
@@ -680,13 +748,19 @@
 	        }
 
 	return;
-        }
+}
 
 
+int my_system(char *command,int timeout,int *early_timeout,char *output,int output_length){
+    return my_system_v2(NRPE_PACKET_VERSION_1, command, NULL, timeout, early_timeout, output, output_length);
+}
 
+#ifndef S_ISREG
+#define S_ISREG(mode)   (((mode) & S_IFMT) == S_IFREG)
+#endif
 
 /* executes a system command via popen(), but protects against timeouts */
-int my_system(char *command,int timeout,int *early_timeout,char *output,int output_length){
+int my_system_v2(int version, char *command,char *largv,int timeout,int *early_timeout,char *output,int output_length){
         pid_t pid;
 	int status;
 	int result;
@@ -696,6 +770,9 @@
 	FILE *fp;
 	int bytes_read=0;
 	time_t start_time,end_time;
+    char command_to_run[MAX_INPUT_BUFFER]="";
+    int i = 0;
+    struct stat sb;
 
 	/* initialize return variables */
 	if(output!=NULL)
@@ -706,6 +783,41 @@
 	if(command==NULL)
 	        return STATE_OK;
 
+    /* trim the command */
+    strip(command);
+
+
+    /* build the command to run */
+    snprintf(command_to_run, MAX_INPUT_BUFFER, "%s", command);
+
+    if(version == NRPE_PACKET_VERSION_2)
+    {
+        /* ensure that the command itself exists */
+        if( stat(command, &sb) != 0)
+        {
+            sprintf(output, "Command requested '%s' doesn't exist\n", command);
+            return STATE_UNKNOWN;
+        }
+        else
+        {
+            if(! S_ISREG(sb.st_mode))
+            {
+                sprintf(output, "Command requested %s isn't a file\n", command);
+                return STATE_UNKNOWN;
+            }
+        }                
+        
+        /* now add all the arguments */
+        strcat(command_to_run, " ");
+        strcat(command_to_run, largv);
+    }
+    if(debug==TRUE)
+        if(version == NRPE_PACKET_VERSION_2)
+            syslog(LOG_DEBUG,"Running command w/ args: %s",command_to_run);
+        else
+            syslog(LOG_DEBUG,"Running command as is: %s",command_to_run);
+
+
 	/* create a pipe */
 	pipe(fd);
 
@@ -751,7 +863,7 @@
 		alarm(timeout);
 
 		/* run the command */
-		fp=popen(command,"r");
+		fp=popen(command_to_run,"r");
 		
 		/* report an error if we couldn't run the command */
 		if(fp==NULL){
@@ -842,7 +954,7 @@
 #endif
 
 	return result;
-        }
+}
 
 
 
@@ -851,7 +963,7 @@
 
 	/* force the child process to exit... */
 	exit(STATE_CRITICAL);
-        }
+}
 
 
 
@@ -924,4 +1036,4 @@
 	        }
 
 	return OK;
-        }
+}
